@@ -114,6 +114,8 @@ my %optionFormats =
 		'e=i',
 	'factor' =>
 		'e=f',
+	'includeUnk' =>
+		'f',
 	'include' =>
 		'i',
 	'cellular' =>
@@ -189,6 +191,7 @@ my %optionDescriptions =
 	'hueGood' => 'Hue (0-360) for "good" scores.',
 	'percentIdentity' => 'Use percent identity for average scores instead of log[10] e-value.',
 	'include' => 'Include a wedge for queries with no hits.',
+	'includeUnk' => 'If any best hits have unknown accessions, force classification to root instead of ignoring them.',
 	'local' => 'Use resources from the local KronaTools installation instead of bundling them with charts (charts will be smaller but will only work on this computer).',
 	'magCol' => 'Column of input files to use as magnitude. If magnitude files are specified, their magnitudes will override those in this column.',
 	'minConfidence' => 'Minimum confidence. Each query sequence will only be added to taxa that were predicted with a confidence score of at least this value.',
@@ -261,7 +264,7 @@ default, the basename of the file will be used.',
 # Global constants #
 ####################
 
-our $version = '2.7.1';
+our $version = '2.8.1';
 my $javascriptVersion = '2.0';
 my $javascript = "src/krona-$javascriptVersion.js";
 my $hiddenImage = 'img/hidden.png';
@@ -915,7 +918,7 @@ sub classifyBlast
 	# taxonomically classifies BLAST results based on LCA (or random selection)
 	# of 'best' hits.
 	#
-	# Options used: bitScore, factor, include, percentIdentity, random, score
+	# Options used: bitScore, factor, include, percentIdentity, random, score, includeUnk
 	
 	my # parameters
 	(
@@ -1008,21 +1011,32 @@ sub classifyBlast
 			last; # EOF
 		}
 		
-		my $acc = getAccFromSeqID($hitID);
-		
-		if ( ! defined $acc )
-		{
-			$lastQueryID = $queryID;
-			next;
-		}
-		
 		if # this is a 'best' hit if...
 		(
-			$queryID ne $lastQueryID || # new query ID (including null at EOF)
+			$ties == 0 || # first hit
 			$bitScore >= $topScore - $options{'threshold'} || # within score threshold
 			$options{'factor'} && $eVal <= $options{'factor'} * $topEVal # within e-val factor
 		)
 		{
+			my $acc = getAccFromSeqID($hitID);
+			my $newTaxID = getTaxIDFromAcc($acc);
+			
+			if
+			(
+				! defined $acc ||
+				! $options{'includeUnk'} && (! $newTaxID || ! taxIDExists($newTaxID))
+			)
+			{
+				$lastQueryID = $queryID;
+				next;
+			}
+			
+			if ( $ties == 0 )
+			{
+				$topScore = $bitScore;
+				$topEVal = $eVal;
+			}
+			
 			# add score for average
 			#
 			if ( $options{'percentIdentity'} )
@@ -1055,8 +1069,6 @@ sub classifyBlast
 				int(rand($ties)) == 0 # randomly chosen to replace other hit
 			)
 			{
-				my $newTaxID = getTaxIDFromAcc($acc);
-				
 				if ( ! $newTaxID || ! taxIDExists($newTaxID) )
 				{
 					$newTaxID = 1;
@@ -1073,12 +1085,6 @@ sub classifyBlast
 			}
 		}
 		
-		if ( $queryID ne $lastQueryID )
-		{
-			$topScore = $bitScore;
-			$topEVal = $eVal;
-		}
-		
 		$lastQueryID = $queryID;
 	}
 	
@@ -1218,6 +1224,11 @@ sub getTaxInfo
 	
 	$tax = int($tax);
 	
+	if ( $tax == 0 )
+	{
+		return ('', '', '', '', '');
+	}
+	
 	if ( defined $taxInfoByID{$tax} )
 	{
 		return @{$taxInfoByID{$tax}};
@@ -1822,6 +1833,11 @@ sub taxLowestCommonAncestor
 	#
 	foreach my $node ( @nodes )
 	{
+		if ( $node == 1)
+		{
+		    return 1; # early out if any nodes are root
+		}
+		
 		while ( getTaxDepth($node) > $minDepth )
 		{
 			$node = getTaxParent($node);
